@@ -131,7 +131,8 @@
 		     ]).
 :- predicate_options(system:create_prolog_flag/3, 3,
 		     [ access(oneof([read_write,read_only])),
-		       type(oneof([boolean,atom,integer,float,term]))
+		       type(oneof([boolean,atom,integer,float,term])),
+		       keep(boolean)
 		     ]).
 :- predicate_options(system:qsave_program/2, 2,
 		     [ local(nonneg),
@@ -151,6 +152,7 @@
 :- predicate_options(system:thread_create/3, 3,
 		     [ alias(atom),
 		       at_exit(callable),
+		       inherit_from(any),
 		       detached(boolean),
 		       global(nonneg),
 		       local(nonneg),
