@@ -1,11 +1,10 @@
-/*  $Id$
-
-    Part of SWI-Prolog
+/*  Part of SWI-Prolog
 
     Author:        Jan Wielemaker
-    E-mail:        wielemak@science.uva.nl
+    E-mail:        J.Wielemaker@cs.vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (C): 1985-2007, University of Amsterdam
+    Copyright (C): 1985-2012, University of Amsterdam
+			      VU University Amsterdam
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -19,7 +18,7 @@
 
     You should have received a copy of the GNU General Public
     License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
     As a special exception, if you link this library with other files,
     compiled with a Free Software compiler, to produce an executable, this
@@ -42,6 +41,7 @@
 	    current_setting/1,		% Module:Name
 	    setting_property/2,		% ?Setting, ?Property
 	    list_settings/0,
+	    list_settings/1,		% +Module
 
 	    convert_setting_text/3	% +Type, +Text, -Value
 	  ]).
@@ -49,6 +49,8 @@
 :- use_module(library(broadcast)).
 :- use_module(library(debug)).
 :- use_module(library(option)).
+:- use_module(library(arithmetic)).
+:- set_prolog_flag(generate_debug_info, false).
 
 /** <module> Setting management
 
@@ -57,7 +59,7 @@ applications. Applications define settings  in   one  or  multiple files
 using the directive setting/4 as illustrated below:
 
 ==
-:- use_module(library(setting)).
+:- use_module(library(settings)).
 
 :- setting(version, atom,   '1.0', 'Current version').
 :- setting(timeout, number,    20, 'Timeout in seconds').
@@ -94,6 +96,8 @@ access, loading and saving of settings.
 	current_setting(:),
 	restore_setting(:).
 
+:- predicate_options(load_settings/2, 2, [undefined(oneof([load,error]))]).
+
 curr_setting(Name, Module, Type, Default, Comment, Src) :-
 	current_setting(Name, Module, Type, Default0, Comment, Src),
 	(   st_default(Name, Module, Default1)
@@ -225,7 +229,7 @@ eval_default(env(Name), _, Type, Value) :- !,
 eval_default(env(Name, Default), _, Type, Value) :- !,
 	(   getenv(Name, TextValue)
 	->  convert_setting_text(Type, TextValue, Val)
-	;   Value = Default
+	;   Val = Default
 	),
 	assert(setting_cache(env(Name), Type, Val)),
 	Value = Val.
@@ -235,7 +239,7 @@ eval_default(setting(Name), Module, Type, Value) :- !,
 	must_be(Type, Value).
 eval_default(Expr, _, Type, Value) :-
 	numeric_type(Type, Basic), !,
-	Val0 is Expr,
+	arithmetic_expression_value(Expr, Val0),
 	(   Basic == float
 	->  Val is float(Val0)
 	;   Basic = integer
@@ -341,16 +345,19 @@ set_setting(QName, Value) :-
 	must_be(atom, Name),
 	(   curr_setting(Name, Module, Type, Default0, _Comment, _Src),
 	    eval_default(Default0, Module, Type, Default)
-	->  (   Value == Default
+	->  setting(Module:Name, Old),
+	    (   Value == Default
 	    ->	retract_setting(Module:Name)
 	    ;	st_value(Name, Module, Value)
 	    ->	true
 	    ;	check_type(Type, Value)
-	    ->	setting(Module:Name, Old),
-	        retract_setting(Module:Name),
-	        assert_setting(Module:Name, Value),
-		broadcast(settings(changed(Module:Name, Old, Value))),
-		clear_setting_cache	% might influence dependent settings.
+	    ->	retract_setting(Module:Name),
+	        assert_setting(Module:Name, Value)
+	    ),
+	    (	Old == Value
+	    ->	true
+	    ;	broadcast(settings(changed(Module:Name, Old, Value))),
+		clear_setting_cache	% might influence dependent settings
 	    )
 	;   existence_error(setting, Name)
 	).
@@ -462,7 +469,7 @@ load_settings(Setting, In, Options) :-
 
 read_setting(In, Term) :-
 	read_term(In, Term,
-		  [ errors(dec10)
+		  [ syntax_errors(dec10)
 		  ]).
 
 %%	store_setting(Term, +Options)
@@ -525,7 +532,7 @@ save_setting(Out, Module:Name) :-
 		 debug(setting, '~w <-> ~w~n', [DefValue, Value]),
 	         DefValue =@= Value
 	       )
-	->  format(Out, '~n%	~w~n', [Comment]),
+	->  format(Out, '~n%\t~w~n', [Comment]),
 	    format(Out, 'setting(~q:~q, ~q).~n', [Module, Name, Value])
 	;   true
 	).
@@ -571,18 +578,33 @@ setting_property(default(Default), M, Type, Default0, _, _) :-
 setting_property(comment(Comment), _, _, _, Comment, _).
 setting_property(source(Src), _, _, _, _, Src).
 
-%%	list_settings
+%%	list_settings is det.
+%%	list_settings(+Module) is det.
 %
-%	List settings to =current_output=.
+%	List settings to =current_output=. The   second  form only lists
+%	settings on the matching module.
+%
+%	@tbd	Compute the required column widths
 
 list_settings :-
-	format('~`=t~72|~n'),
-	format('~w~t~20| ~w~w~t~40| ~w~n', ['Name', 'Value (*=modified)', '', 'Comment']),
-	format('~`=t~72|~n'),
-	forall(current_setting(Module:Setting),
-	       list_setting(Module:Setting)).
+	list_settings(_).
 
-list_setting(Module:Name) :-
+list_settings(Spec) :-
+	spec_term(Spec, Term),
+	TS1 = 25,
+	TS2 = 40,
+	format('~`=t~72|~n'),
+	format('~w~t~*| ~w~w~t~*| ~w~n',
+	       ['Name', TS1, 'Value (*=modified)', '', TS2, 'Comment']),
+	format('~`=t~72|~n'),
+	forall(current_setting(Term),
+	       list_setting(Term, TS1, TS2)).
+
+spec_term(M:S, M:S) :- !.
+spec_term(M, M:_).
+
+
+list_setting(Module:Name, TS1, TS2) :-
 	curr_setting(Name, Module, Type, Default0, Comment, _Src),
 	eval_default(Default0, Module, Type, Default),
 	setting(Module:Name, Value),
@@ -590,7 +612,7 @@ list_setting(Module:Name) :-
 	->  Modified = (*)
 	;   Modified = ''
 	),
-        format('~w~t~20| ~q~w~t~40| ~w~n', [Module:Name, Value, Modified, Comment]).
+        format('~w~t~*| ~q~w~t~*| ~w~n', [Module:Name, TS1, Value, Modified, TS2, Comment]).
 
 
 		 /*******************************
