@@ -131,7 +131,7 @@ void            rule            (bool, long, long);
 void            ruleaux         (long, long, char);
 long            horizontalmove  (long);
 int             skipnops        (void);
-linetype    *   getline         (void);
+linetype    *   dvi_getline         (void);
 linetype    *   findline        (void);
 unsigned long   num             (int);
 long            snum            (int);
@@ -156,7 +156,7 @@ void            rule            ();
 void            ruleaux         ();
 long            horizontalmove  ();
 int             skipnops        ();
-linetype    *   getline         ();
+linetype    *   dvi_getline         ();
 linetype    *   findline        ();
 unsigned long   num             ();
 long            snum            ();
@@ -303,7 +303,7 @@ void initpage()
     h = 0L;  v = 0L;                        /* initialize coordinates   */
     x = 0L;  w = 0L;  y = 0L;  z = 0L;      /* initialize amounts       */
     sptr = 0;                               /* initialize stack         */
-    currentline = getline();                /* initialize list of lines */
+    currentline = dvi_getline();                /* initialize list of lines */
     currentline->vv = 0L;
     firstline   = currentline;
     lastline    = currentline;
@@ -689,7 +689,7 @@ int skipnops()                      /* skips by no-op commands  */
 
 /*----------------------------------------------------------------------------*/
 
-linetype *getline()             /* returns an initialized line-object */
+linetype *dvi_getline()             /* returns an initialized line-object */
 {
     register int  i;
     register linetype *temp;
@@ -706,7 +706,7 @@ linetype *getline()             /* returns an initialized line-object */
     temp->text[i] = '\0';
     return temp;
 
-} /* getline */
+} /* dvi_getline */
 
 /*----------------------------------------------------------------------------*/
 
@@ -716,7 +716,7 @@ linetype *findline()            /* find best fit line were text should go */
     register long topd, botd;
     if (v <= firstline->vv) {                      /* above first line */
         if (firstline->vv - v >= VERTICALEPSILON) {
-            temp = getline();
+            temp = dvi_getline();
             temp->next = firstline;
             firstline->prev = temp;
             temp->vv = v;
@@ -727,7 +727,7 @@ linetype *findline()            /* find best fit line were text should go */
 
     if (v >= lastline->vv) {                       /* below last line */
         if (v - lastline->vv >= VERTICALEPSILON) {
-            temp = getline();
+            temp = dvi_getline();
             temp->prev = lastline;
             lastline->next = temp;
             temp->vv = v;
@@ -750,7 +750,7 @@ linetype *findline()            /* find best fit line were text should go */
             return temp->next;
 
     /* no line fits suitable, generate a new one */
-    currentline = getline();
+    currentline = dvi_getline();
     currentline->next = temp->next;
     currentline->prev = temp;
     temp->next->prev = currentline;
