@@ -267,6 +267,7 @@ static int
 ar_close(struct archive *a, void *cdata)
 { archive_wrapper *ar = cdata;
 
+  PL_release_stream(ar->data);
   if ( ar->close_parent )
   { if ( Sclose(ar->data) != 0 )
     { archive_set_error(ar->archive, errno, "Close failed");
@@ -721,6 +722,7 @@ archive_property(term_t archive, term_t prop, term_t value)
        !PL_get_atom_ex(prop, &pn) )
     return FALSE;
 
+#ifdef HAVE_ARCHIVE_FILTER_COUNT
   if ( pn == ATOM_filter )
   { int i, fcount = archive_filter_count(ar->archive);
     term_t tail = PL_copy_term_ref(value);
@@ -738,6 +740,7 @@ archive_property(term_t archive, term_t prop, term_t value)
     }
     return PL_unify_nil(tail);
   }
+#endif
 
   return FALSE;
 }
@@ -752,6 +755,8 @@ archive_next_header(term_t archive, term_t name)
     return FALSE;
   if ( ar->status == AR_NEW_ENTRY )
     archive_read_data_skip(ar->archive);
+  if ( ar->status == AR_OPENED_ENTRY )
+    return PL_permission_error("next_header", "archive", archive);
 
   while ( (rc=archive_read_next_header(ar->archive, &ar->entry)) == ARCHIVE_OK )
   { if ( PL_unify_wchars(name, PL_ATOM, -1,
@@ -895,15 +900,19 @@ static int
 ar_close_entry(void *handle)
 { archive_wrapper *ar = handle;
 
-  if ( ar->status == AR_OPENED_ENTRY )
-  { PL_unregister_atom(ar->symbol);
-    ar->status = AR_CLOSED_ENTRY;
-  }
   if ( ar->closed_archive )
-  { archive_read_close(ar->archive);
+  { struct archive *a = ar->archive;
+
+    ar->archive = NULL;
     ar->entry = NULL;
     ar->archive = NULL;
     ar->symbol = 0;
+
+    archive_read_close(a);
+  }
+  if ( ar->status == AR_OPENED_ENTRY )
+  { PL_unregister_atom(ar->symbol);
+    ar->status = AR_CLOSED_ENTRY;
   }
 
   return 0;
