@@ -17,7 +17,7 @@
 
     You should have received a copy of the GNU Lesser General Public
     License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
     As a special exception, if you link this library with other files,
     compiled with a Free Software compiler, to produce an executable, this
@@ -33,10 +33,13 @@
 	    http_delete_handler/1,	% +Path
 	    http_reply_file/3,		% +File, +Options, +Request
 	    http_redirect/3,		% +How, +Path, +Request
+	    http_404/2,			% +Options, +Request
+	    http_switch_protocol/2,	% :Goal, +Options
 	    http_current_handler/2,	% ?Path, ?Pred
 	    http_current_handler/3,	% ?Path, ?Pred
 	    http_location_by_id/2,	% +ID, -Location
 	    http_link_to_id/3,		% +ID, +Parameters, -HREF
+	    http_reload_with_parameters/3, % +Request, +Parameters, -HREF
 	    http_safe_file/2		% +Spec, +Options
 	  ]).
 :- use_module(library(option)).
@@ -50,6 +53,15 @@
 :- use_module(library(http/http_header)).
 :- use_module(library(http/thread_httpd)).
 
+:- predicate_options(http_404/2, 1, [index(any)]).
+:- predicate_options(http_reply_file/3, 2,
+		     [ cache(boolean),
+		       mime_type(any),
+		       pass_to(http_safe_file/2, 2)
+		     ]).
+:- predicate_options(http_safe_file/2, 2, [unsafe(boolean)]).
+:- predicate_options(http_switch_protocol/2, 2, []).
+
 /** <module> Dispatch requests in the HTTP server
 
 This module can be placed between   http_wrapper.pl  and the application
@@ -60,7 +72,7 @@ timeout handling and user authentication.  The typical setup is:
 ==
 server(Port, Options) :-
 	http_server(http_dispatch,
-		    [ port(Port),
+		    [ port(Port)
 		    | Options
 		    ]).
 
@@ -110,7 +122,15 @@ write_index(Request) :-
 %		* prefix
 %		Call Pred on any location that is a specialisation of
 %		Path.  If multiple handlers match, the one with the
-%		longest path is used.
+%		longest path is used.  Options defined with a prefix
+%		handler are the default options for paths that start
+%		with this prefix.  Note that the handler acts as a
+%		fallback handler for the tree below it:
+%
+%		  ==
+%		  :- http_handler(/, http_404([index('index.html')]),
+%				  [spawn(my_pool),prefix]).
+%		  ==
 %
 %		* priority(+Integer)
 %		If two handlers handle the same path, the one with the
@@ -148,7 +168,8 @@ write_index(Request) :-
 :- meta_predicate
 	http_handler(+, :, +),
 	http_current_handler(?, :),
-	http_current_handler(?, :, ?).
+	http_current_handler(?, :, ?),
+	http_switch_protocol(2, +).
 
 http_handler(Path, Pred, Options) :-
 	strip_module(Pred, M, P),
@@ -180,7 +201,7 @@ system:term_expansion((:- http_handler(Path, Pred, Options)), Clause) :-
 
 http_delete_handler(id(Id)) :- !,
 	clause(handler(_Path, _:Pred, _, Options), true, Ref),
-	functor(Pred, _, DefID),
+	functor(Pred, DefID, _),
 	option(id(Id0), Options, DefID),
 	Id == Id0,
 	erase(Ref),
@@ -408,14 +429,45 @@ location_by_id_raw(ID, Location, Priority) :-
 %
 %	HREF is a link on the local server to a handler with given ID,
 %	passing the given Parameters.
+%
+%	@arg Parameters is one of
+%
+%		- path_postfix(File) to pass a single value as the last
+%		  segment of the HTTP location (path). This way of
+%		  passing a parameter is commonly used in REST APIs.
+%		- A list of search parameters for a =GET= request.
 
+http_link_to_id(HandleID, path_postfix(File), HREF) :- !,
+	http_location_by_id(HandleID, HandlerLocation),
+	uri_encoded(path, File, EncFile),
+	directory_file_path(HandlerLocation, EncFile, Location),
+	uri_data(path, Components, Location),
+	uri_components(HREF, Components).
 http_link_to_id(HandleID, Parameters, HREF) :-
+	must_be(list, Parameters),
 	http_location_by_id(HandleID, Location),
 	uri_data(path, Components, Location),
 	uri_query_components(String, Parameters),
 	uri_data(search, Components, String),
 	uri_components(HREF, Components).
 
+%%	http_reload_with_parameters(+Request, +Parameters, -HREF) is det.
+%
+%	Create a request on the current handler with replaced search
+%	parameters.
+
+http_reload_with_parameters(Request, NewParams, HREF) :-
+	memberchk(path(Path), Request),
+	(   memberchk(search(Params), Request)
+	->  true
+	;   Params = []
+	),
+	merge_options(NewParams, Params, AllParams),
+	uri_query_components(Search, AllParams),
+	uri_data(path, Data, Path),
+	uri_data(search, Data, Search),
+	uri_components(HREF, Data).
+
 
 %	hook into html_write:attribute_value//1.
 
@@ -650,7 +702,8 @@ http_safe_file(Name) :-
 
 safe_name(Name, _) :-
 	must_be(atom, Name),
-	\+ unsafe_name(Name), !.
+	prolog_to_os_filename(FileName, Name),
+	\+ unsafe_name(FileName), !.
 safe_name(_, Spec) :-
 	permission_error(read, file, Spec).
 
@@ -672,7 +725,7 @@ unsafe_name(Name) :- sub_atom(Name, _, _, 0, '/..').
 %			    []).
 %	    ==
 %
-%	@param How is one of =moved=, =moved_temporary= or =see_also=
+%	@param How is one of =moved=, =moved_temporary= or =see_other=
 %	@param To is an atom, a aliased path as defined by
 %	http_absolute_location/3. or a term location_by_id(Id). If To is
 %	not absolute, it is resolved relative to the current location.
@@ -683,11 +736,65 @@ http_redirect(How, To, Request) :-
 	;   memberchk(path(Base), Request),
 	    http_absolute_location(To, URL, [relative_to(Base)])
 	),
-	must_be(oneof([moved, moved_temporary, see_also]), How),
+	must_be(oneof([moved, moved_temporary, see_other]), How),
 	Term =.. [How,URL],
 	throw(http_reply(Term)).
 
 
+%%	http_404(+Options, +Request) is det.
+%
+%	Reply using an "HTTP  404  not   found"  page.  This  handler is
+%	intended as fallback handler  for   _prefix_  handlers.  Options
+%	processed are:
+%
+%	    * index(Location)
+%	    If there is no path-info, redirect the request to
+%	    Location using http_redirect/3.
+%
+%	@error http_reply(not_found(Path))
+
+http_404(Options, Request) :-
+	option(index(Index), Options),
+	\+ ( memberchk(path_info(PathInfo), Request),
+	     PathInfo \== ''
+	   ), !,
+	http_redirect(moved, Index, Request).
+http_404(_Options, Request) :-
+	memberchk(path(Path), Request),
+	throw(http_reply(not_found(Path))).
+
+
+%%	http_switch_protocol(:Goal, +Options)
+%
+%	Send an =|HTTP 101 Switching   Protocols"|= reply. After sending
+%	the  reply,  the  HTTP  library    calls   call(Goal,  InStream,
+%	OutStream), where InStream and OutStream are  the raw streams to
+%	the HTTP client. This allows the communication to continue using
+%	an an alternative protocol.
+%
+%	If Goal fails or throws an exception,  the streams are closed by
+%	the server. Otherwise  Goal  is   responsible  for  closing  the
+%	streams. Note that  Goal  runs  in   the  HTTP  handler  thread.
+%	Typically, the handler should be   registered  using the =spawn=
+%	option if http_handler/3 or Goal   must  call thread_create/3 to
+%	allow the HTTP worker to return to the worker pool.
+%
+%	The streams use binary  (octet)  encoding   and  have  their I/O
+%	timeout set to the server  timeout   (default  60  seconds). The
+%	predicate set_stream/2 can  be  used   to  change  the encoding,
+%	change or cancel the timeout.
+%
+%	This predicate interacts with the server  library by throwing an
+%	exception.
+%
+%	@param	Options is reserved for future extensions.  It must be
+%		initialised to the empty list ([]).
+%	@throws	http_reply(switch_protocol(Goal, Options))
+
+http_switch_protocol(Goal, Options) :-
+	throw(http_reply(switch_protocol(Goal, Options))).
+
+
 		 /*******************************
 		 *	  PATH COMPILATION	*
 		 *******************************/
@@ -777,7 +884,8 @@ add_paths_tree([path(Path, Action, Options)|T], Tree0, Tree) :-
 
 plain_path(Path, Action, Options) :-
 	handler(Spec, Action, false, Options),
-	http_absolute_location(Spec, Path, []).
+	catch(http_absolute_location(Spec, Path, []), E,
+	      (print_message(error, E), fail)).
 
 
 %%	add_path_tree(+Path, +Action, +Options, +Tree0, -Tree) is det.
