@@ -17,7 +17,7 @@
 
     You should have received a copy of the GNU General Public
     License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
     As a special exception, if you link this library with other files,
     compiled with a Free Software compiler, to produce an executable, this
@@ -35,7 +35,7 @@
 :- use_module(library(settings)).
 :- use_module(library(broadcast)).
 
-:- setting(http:logfile, atom, 'httpd.log',
+:- setting(http:logfile, callable, 'httpd.log',
 	   'File in which to log HTTP requests').
 
 /** <module> HTTP Logging module
@@ -48,7 +48,7 @@ library(settings) for details.
 The  level  of  logging  can  modified  using  the  multifile  predicate
 http_log:nolog/1 to hide HTTP  request  fields   from  the  logfile  and
 http_log:password_field/1   to   hide   passwords   from   HTTP   search
-specifications (e.g. =|/topsecret?password=secret|=.
+specifications (e.g. =|/topsecret?password=secret|=).
 */
 
 :- multifile
@@ -77,34 +77,51 @@ http_message(request_finished(Id, Code, Status, CPU, Bytes)) :- !,
 		 *******************************/
 
 :- dynamic
-	log_stream/1.
+	log_stream/2.			% Stream, TimeTried
 
 %%	http_log_stream(-Stream) is semidet.
 %
-%	Returns handle to open logfile. Fails if no logfile is open and
-%	none is defined.
+%	True when Stream is a stream to  the opened HTTP log file. Opens
+%	the log file in =append= mode if the   file is not yet open. The
+%	log file is determined  from   the  setting =|http:logfile|=. If
+%	this setting is set  to  the   empty  atom  (''), this predicate
+%	fails.
+%
+%	If  a  file  error  is  encountered,   this  is  reported  using
+%	print_message/2, after which this predicate silently fails.
 
 http_log_stream(Stream) :-
-	log_stream(Stream), !,
+	log_stream(Stream, _Opened), !,
 	Stream \== [].
+http_log_stream([]) :-
+	setting(http:logfile, ''), !,
+	get_time(Now),
+	assert(log_stream([], Now)).
 http_log_stream(Stream) :-
-	setting(http:logfile, File),
-	File \== '', !,
+	setting(http:logfile, Term),
+	catch(absolute_file_name(Term, File,
+				 [ access(append)
+				 ]), E, open_error(E)),
 	with_mutex(http_log,
-		   (   open(File, append, Stream,
-			    [ close_on_abort(false),
-			      encoding(utf8),
-			      buffer(line)
-			    ]),
+		   (   catch(open(File, append, Stream,
+				  [ close_on_abort(false),
+				    encoding(utf8),
+				    buffer(line)
+				  ]), E, open_error(E)),
 		       get_time(Time),
 		       format(Stream,
 			      'server(started, ~0f).~n',
 			      [ Time ]),
-		       assert(log_stream(Stream)),
+		       assert(log_stream(Stream, Time)),
 		       at_halt(close_log(stopped))
 		   )).
-http_log_stream(_) :-
-	assert(log_stream([])).
+
+open_error(E) :-
+	print_message(error, E),
+	get_time(Now),
+	assert(log_stream([], Now)),
+	fail.
+
 
 %%	http_log_close(+Reason) is det.
 %
@@ -125,12 +142,12 @@ http_log_close(Reason) :-
 	with_mutex(http_log, close_log(Reason)).
 
 close_log(Reason) :-
-	retract(log_stream(Stream)), !,
-	(   Stream == []
-	->  true
-	;   get_time(Time),
+	retract(log_stream(Stream, _Opened)), !,
+	(   is_stream(Stream)
+	->  get_time(Time),
 	    format(Stream, 'server(~q, ~0f).~n', [ Reason, Time ]),
 	    close(Stream)
+	;   true
 	).
 close_log(_).
 
@@ -247,7 +264,7 @@ log_check_deleted(_).
 
 
 log(Code, ok, Bytes, Id, CPU, Stream) :- !,
-	format(Stream, 'completed(~q, ~2f, ~q, ~q, ok).~n',
+	format(Stream, 'completed(~q, ~q, ~q, ~q, ok).~n',
 	       [ Id, CPU, Bytes, Code ]).
 log(Code, Status, Bytes, Id, CPU, Stream) :-
 	(   map_exception(Status, Term)
@@ -255,7 +272,7 @@ log(Code, Status, Bytes, Id, CPU, Stream) :-
 	;   message_to_string(Status, String),
 	    Term = error(String)
 	),
-	format(Stream, 'completed(~q, ~2f, ~q, ~q, ~q).~n',
+	format(Stream, 'completed(~q, ~q, ~q, ~q, ~q).~n',
 	       [ Id, CPU, Bytes, Code, Term ]).
 
 map_exception(http_reply(Reply), Reply).
