@@ -273,6 +273,7 @@ open_transaction(rdf_db *db,
 
   q->type = Q_TRANSACTION;
   q->transaction = ti->queries.transaction;
+  q->reindex_gen = GEN_MAX;		/* should not get this down */
 
   if ( ss && ss != SNAPSHOT_ANONYMOUS )
   { int ss_tid = snapshot_thread(ss);
@@ -838,7 +839,8 @@ discard_transaction(query *q)
 
 					/* revert creation of new */
     if ( is_wr_transaction_gen(q, t->lifespan.born) )
-    { t->lifespan.died = GEN_PREHIST;
+    { t = deref_triple(db, t);
+      t->lifespan.died = GEN_PREHIST;
       erase_triple(db, t, q);
     }
   }
@@ -850,7 +852,9 @@ discard_transaction(query *q)
 
 					/* revert deletion of old */
     if ( is_wr_transaction_gen(q, t->lifespan.died) )
-    { t->lifespan.died = GEN_MAX;
+    { t = deref_triple(db, t);
+
+      t->lifespan.died = GEN_MAX;
     }
   }
 
@@ -862,12 +866,15 @@ discard_transaction(query *q)
 
 					/* revert deletion of old */
     if ( is_wr_transaction_gen(q, to->lifespan.died) )
-    { to->lifespan.died = GEN_MAX;
+    { to = deref_triple(db, to);
+
+      to->lifespan.died = GEN_MAX;
     }
 					/* revert creation of new */
     if ( is_wr_transaction_gen(q, tn->lifespan.born) &&
 	 tn->lifespan.died == gen_max )
-    { tn->lifespan.died = GEN_PREHIST;
+    { tn = deref_triple(db, tn);
+      tn->lifespan.died = GEN_PREHIST;
       erase_triple(db, tn, q);
     }
   }
