@@ -286,6 +286,24 @@ typedef struct literal
 typedef unsigned int triple_id;		/* Triple identifier */
 #endif
 
+#ifdef TRIPLE_MAGIC
+
+typedef enum
+{ T_CHAINED1 = TRIPLE_MAGIC,		/* Added to hash-chains */
+  T_CHAINED2,				/* Added to hash as reindexed */
+  T_REINDEXED,				/* Reindexed (waiting for GC) */
+  T_LINGERING,				/* waiting to be freed */
+  T_FREED				/* freed */
+} triple_status;
+
+#define TMAGIC(t, s) (t->magic = s)
+
+#else
+
+#define TMAGIC(t, s) (void)0
+
+#endif
+
 typedef struct triple
 { lifespan	lifespan;		/* Start and end generation */
   atom_id	subject_id;
@@ -327,7 +345,11 @@ typedef struct triple
   unsigned	loaded : 1;		/* for EV_ASSERT_LOAD */
   unsigned	erased : 1;		/* Consistency of erased */
   unsigned	lingering : 1;		/* Deleted; waiting for GC */
+  unsigned	has_reindex_prev : 1;	/* some ->reindexed points to me */
 					/* Total: 32 */
+#ifdef TRIPLE_MAGIC
+  triple_status	magic;
+#endif
 } triple;
 
 
@@ -424,7 +446,8 @@ typedef struct rdf_db
 #endif
   size_t	created;		/* #triples created */
   size_t	erased;			/* #triples erased */
-  gen_t		reindexed;		/* #triples reindexed (gc_hash_chain) */
+  size_t	reindexed;		/* #triples reindexed (gc_hash_chain) */
+  size_t	lingering;		/* #triples lingering to be freed */
   size_t	indexed[16];		/* Count calls (2**4 possible indices) */
   resource_db	resources;		/* admin of used resources */
   pred_hash	predicates;		/* Predicate table */
