@@ -1,11 +1,10 @@
-/*  $Id$
+/*  Part of SWI-Prolog
 
-    Part of SWI-Prolog
-
-    Author:        Jan Wielemaker
-    E-mail:        jan@swi.psy.uva.nl
+    Author:        Jan van der Steen, Matt Lilley and Jan Wielemaker,
+    E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (C): 2004, SWI-Prolog Foundation
+    Copyright (C): 2004-2014, SWI-Prolog Foundation
+			      VU University Amsterdam
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -19,7 +18,7 @@
 
     You should have received a copy of the GNU Lesser General Public
     License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
     As a special exception, if you link this library with other files,
     compiled with a Free Software compiler, to produce an executable, this
@@ -43,8 +42,10 @@
             ssl_accept/3,               % +Config, -Socket, -Peer
             ssl_open/3,                 % +Config, -Read, -Write
             ssl_open/4,                 % +Config, +Socket, -Read, -Write
-            ssl_negotiate/5,            % +Config, +PlainRead, +PlainWrite, -SSLRead, -SSLWrite
-	    ssl_exit/1			% +Config
+            ssl_negotiate/5,            % +Config, +PlainRead, +PlainWrite,
+					%          -SSLRead,   -SSLWrite
+	    ssl_exit/1,			% +Config
+            ssl_session/2               % +Stream, -Session
 	  ]).
 :- use_module(library(socket)).
 :- use_module(library(error)).
@@ -56,8 +57,25 @@
 	ssl_init(-, +, :),
 	ssl_context(+, -, :).
 
+:- predicate_options(ssl_context/3, 3,
+		     [ host(atom),
+		       port(integer),
+		       certificate_file(atom),
+		       key_file(atom),
+		       password(any),
+		       pem_password_hook(callable),
+		       cacert_file(atom),
+		       cert_verify_hook(callable),
+		       cert(boolean),
+		       peer_cert(boolean),
+		       close_parent(boolean)
+		     ]).
+:- predicate_options(ssl_init/3, 3, [pass_to(ssl_context/3, 3)]).
 
-ssl_context(Role, SSL, Options) :-
+/** <module> Secure Socket Layer library
+*/
+
+ssl_context(Role, SSL, Options) :-	% Prolog to exploit meta-predicate
 	'_ssl_context'(Role, SSL, Options).
 
 /*
@@ -76,7 +94,10 @@ ssl_init2(server, SSL, Options) :-
 	tcp_setopt(Socket, reuseaddr),
         tcp_bind(Socket, Port),
         tcp_listen(Socket, 5),
-        ssl_context(server, SSL, Options),
+        catch(ssl_context(server, SSL, Options),
+              Exception,
+              ( tcp_close_socket(Socket),
+                throw(Exception))),
         Socket = '$socket'(S),
         ssl_put_socket(SSL, S).
 ssl_init2(client, SSL, Options) :-
@@ -86,7 +107,10 @@ ssl_init2(client, SSL, Options) :-
         tcp_socket(Socket),
 	tcp_setopt(Socket, reuseaddr),
         tcp_connect(Socket, Host:Port),
-        ssl_context(client, SSL, Options),
+        catch(ssl_context(client, SSL, Options),
+              Exception,
+              ( tcp_close_socket(Socket),
+                throw(Exception))),
         Socket = '$socket'(S),
         ssl_put_socket(SSL, S).
 
