@@ -418,6 +418,7 @@ unify_asn1_time(term_t term, ASN1_TIME *time)
      memcpy(pbuffer, source, 10);
      pbuffer += 10;
      source += 10;
+     length -= 10;
   } else
   { if (length < 13)
      {  ssl_deb(2, "Unable to parse time - expected at least 13 chars, not %d", length);
@@ -427,6 +428,7 @@ unify_asn1_time(term_t term, ASN1_TIME *time)
      memcpy(pbuffer, source, 10);
      pbuffer += 10;
      source += 10;
+     length -= 10;
   }
   /* Next find end of string */
   if ((*source == 'Z') || (*source == '-') || (*source == '+'))
@@ -448,7 +450,7 @@ unify_asn1_time(term_t term, ASN1_TIME *time)
   if (*source == 'Z')
      lSecondsFromUTC = 0;
   else
-  { if ((*source != '+') && (source[5] != '-'))
+  { if ( length < 6 || (*source != '+' && source[5] != '-') )
      {  ssl_deb(2, "Unable to parse time. Missing UTC offset");
         return FALSE;
      }
@@ -551,13 +553,21 @@ unify_name(term_t term, X509_NAME* name)
   for (ni = 0; ni < X509_NAME_entry_count(name); ni++)
   { X509_NAME_ENTRY* e = X509_NAME_get_entry(name, ni);
     ASN1_STRING* entry_data = X509_NAME_ENTRY_get_data(e);
-    if (!(PL_unify_list(list, item, list) &&
-          PL_unify_term(item,
-                        PL_FUNCTOR, FUNCTOR_equals2,
-                        PL_CHARS, OBJ_nid2sn(OBJ_obj2nid(e->object)),
-                        PL_NCHARS, (size_t)entry_data->length, entry_data->data)
-           ))
-       return FALSE;
+    unsigned char *utf8_data;
+    int rc;
+
+    if ( ASN1_STRING_to_UTF8(&utf8_data, entry_data) < 0 )
+      return PL_resource_error("memory");
+
+    rc = ( PL_unify_list(list, item, list) &&
+	   PL_unify_term(item,
+			 PL_FUNCTOR, FUNCTOR_equals2,
+			 PL_CHARS, OBJ_nid2sn(OBJ_obj2nid(e->object)),
+			 PL_UTF8_CHARS, utf8_data)
+	 );
+    OPENSSL_free(utf8_data);
+    if ( !rc )
+      return FALSE;
   }
   return PL_unify_nil(list);
 }
