@@ -19,7 +19,7 @@
 
     You should have received a copy of the GNU Lesser General Public
     License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 #include <h/kernel.h>
@@ -52,13 +52,13 @@ initialiseConnection(Connection c, Graphical from, Graphical to,
   }
 
   copyLine((Line) c, link->line);
-  
+
   assign(c, link,        link);
   assign(c, from_handle, isDefault(from_handle) ? (Name) NIL : from_handle);
   assign(c, to_handle,   isDefault(to_handle)   ? (Name) NIL : to_handle);
   assign(c, fixed_from,  isDefault(from_handle) ? OFF : ON);
   assign(c, fixed_to,    isDefault(to_handle)   ? OFF : ON);
-  
+
   return relateConnection(c, from, to);
 }
 
@@ -237,11 +237,13 @@ findbest:
 #define FindBestHandle \
   { Handle h = cell->value; \
     Int hx, hy; \
+    double dx, dy; \
     if ( h->kind != kind ) \
       continue; \
     getXYHandle(h, gr, dev, &hx, &hy); \
     X = valInt(hx); Y = valInt(hy); \
-    D = isqrt((x-X)*(x-X) + (y-Y)*(y-Y)); \
+    dx = (double)(x-X); dy = (double)(y-Y); \
+    D = rdouble(sqrt(dx*dx+dy*dy)); \
     DC = distanceLineToPoint_int(x, y, X, Y, cx, cy); \
     if ((D + DC < bestd + bestdc) || found == FAIL) \
     { bestd = D; \
@@ -405,7 +407,7 @@ eventConnection(Connection c, EventObj ev)
     Cell cell;
 
     TRY( recognisers = getAllRecognisersGraphical(c->link->line, OFF) );
-  
+
     for_cell(cell, recognisers)
       if ( qadSendv(cell->value, NAME_event, 1, (Any*)&ev) )
 	succeed;
@@ -484,7 +486,7 @@ static getdecl get_connection[] =
 #define rc_connection NULL
 /*
 static classvardecl rc_connection[] =
-{ 
+{
 };
 */
 
