@@ -19,7 +19,7 @@
 
     You should have received a copy of the GNU Lesser General Public
     License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 #include <h/kernel.h>
@@ -128,7 +128,7 @@ recognised:
     ~t	    Figure	Translate to XY of figure
     ~T	    Graphical	Set texture to texture of graphical
     ~C	    Graphical	Output colour of the graphical
-    ~N	    Name 	print text of name
+    ~N	    Name	print text of name
     ~S	    StringObj	Output text of StringObj with postscript escapes
     ~O	    Object	Output comment to start O
     ~P	    Int, Image	Output pattern of grayscale image with depth Int
@@ -149,7 +149,7 @@ _output(char *fm, va_list args)
     { case '\\':
 	switch(*++fm)				/* \ escapes */
 	{ case 'n':	ps_put_char('\n');
-	  		continue;
+			continue;
 	  case '\\':	ps_put_char('\\');
 			continue;
 	  case '\0':	ps_put_char('\\');
@@ -158,7 +158,7 @@ _output(char *fm, va_list args)
       case '~':
 	switch(*++fm)				/* ~ escapes */
 	{ case '~':	ps_put_char('~');
-    			continue;
+			continue;
 	  case '\0':	ps_put_char('~');
 			return;
 	  case 's':	putString(va_arg(args, char *));
@@ -169,9 +169,15 @@ _output(char *fm, va_list args)
 	  case 'D':	sprintf(tmp, "%d", va_arg(args, int));
 			putString(tmp);
 			continue;
-	  case 'f':	sprintf(tmp, "%.2f", va_arg(args, double) );
+	  case 'f':   { char *s;
+
+			sprintf(tmp, "%.2f", va_arg(args, double));
+			for(s=tmp; *s; s++)
+			  if ( *s != '-' && !isalnum(*s) )
+			    *s = '.';
 			putString(tmp);
 			continue;
+		      }
 	  case 'm':   { Graphical gr = va_arg(args, Graphical);
 			sprintf(tmp, INTPTR_FORMAT " " INTPTR_FORMAT " moveto",
 				valInt(gr->area->x),
@@ -296,7 +302,7 @@ ps_output(char *fm, ...)
 static void
 ps_put_string(String s)
 { wint_t c;
-  int i, size = s->size;
+  int i, size = s->s_size;
 
   ps_put_char('(');
 
@@ -313,7 +319,7 @@ ps_put_string(String s)
 			{ ps_put_char(c);
 			} else
 			{ char tmp[8];
-			  sprintf(tmp, "\\%03o", c);
+			  sprintf(tmp, "\\%03o", (unsigned int)c);
 			  putString(tmp);
 			}
     }
@@ -570,7 +576,7 @@ actual object.
 #define PSMACRO(a, b, r) { a, b, r }
 
 typedef struct
-{ Name 		name;
+{ Name		name;
   const char   *def;
   const char   *required;
 } psmacro;
@@ -1652,7 +1658,7 @@ status
 drawPostScriptText(TextObj t, Name hb)
 { String s = &t->string->data;
 
-  if ( s[0].size > 0 )			/* i.e. non-empty */
+  if ( s[0].s_size > 0 )		/* i.e. non-empty */
   { int x, y, w;
     int b = valInt(t->border);
 
@@ -1705,7 +1711,7 @@ drawPostScriptText(TextObj t, Name hb)
 	flags |= TXT_UNDERLINED;
 
       if ( Wrapped(t) )
-      { LocalString(buf, s->iswide, s->size + MAX_WRAP_LINES);
+      { LocalString(buf, s->s_iswide, s->s_size + MAX_WRAP_LINES);
 
 	str_format(buf, s, valInt(t->margin), t->font);
 	ps_string(buf, t->font, x+b, y+b, w-2*b, t->format, flags);
