@@ -19,7 +19,7 @@
 
     You should have received a copy of the GNU Lesser General Public
     License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 #ifndef XPCE_H_INCLUDED
@@ -32,7 +32,7 @@
 
 #ifndef __pce_export
 #if defined(PCE_INCLUDED) && (defined(WIN32) || defined(__CYWIN32__))
-#define __pce_export _declspec(dllexport)
+#define __pce_export __declspec(dllexport)
 #else
 #define __pce_export extern
 #endif /*PCE_INCLUDED*/
