@@ -22,11 +22,6 @@
     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
-#ifdef __CYGWIN__
-#define __USE_W32_SOCKETS
-#define HAVE_WINSOCK 1
-#endif
-
 #include <md.h>				/* get HAVE_'s */
 #if defined(HAVE_SOCKET) || defined(HAVE_WINSOCK)
 
@@ -174,10 +169,12 @@ closeAllSockets(int status)
 }
 
 
+#ifdef SIGPIPE
 static void
 sigPipeSocket(void)
 { errorPce(PCE, NAME_brokenPipe, 0);
 }
+#endif
 
 
 static void
@@ -237,7 +234,7 @@ initialiseSocket(Socket s, Any address, Name domain)
 
   assign(s, domain,	    domain);
   assign(s, address,	    address);
-  assign(s, status, 	    NAME_idle);
+  assign(s, status,	    NAME_idle);
 
   succeed;
 }
@@ -483,7 +480,7 @@ bindSocket(Socket s, BoolObj reuse)
 	    ((Tuple)s->address)->second == ZERO) )
       { struct sockaddr_in addr;
 #ifdef __WINDOWS__
-	DWORD len = sizeof(addr);
+	int len = sizeof(addr);
 #else
 	socklen_t len = sizeof(addr);
 #endif
@@ -526,7 +523,7 @@ acceptSocket(Socket s)
 #endif
   { struct sockaddr_in address;
 #ifdef __WINDOWS__
-    DWORD len = sizeof(address);
+    int len = sizeof(address);
 #else
     socklen_t len = sizeof(address);
 #endif
@@ -539,15 +536,17 @@ acceptSocket(Socket s)
       if ( (hp = gethostbyaddr((char *)&address.sin_addr,
 			       sizeof(address.sin_addr),
 			       AF_INET)) )
-	client_address = newObject(ClassTuple,
-				   CtoName(hp->h_name),
-				   toInt(address.sin_port),
-				   EAV);
+	client_address = answerObject(ClassTuple,
+				      CtoName(hp->h_name),
+				      toInt(address.sin_port),
+				      EAV);
       else
 	client_address = NIL;
     }
   }
 
+  (void)client_address;				/* not used right now */
+
   if ( !(s2 = get(s, NAME_clone, EAV)) )
     return errorPce(s, NAME_failedToClone);
 
@@ -679,7 +678,7 @@ getPeerNameSocket(Socket s)
 #endif /*UNIX_DOMAIN_SOCKETS*/
   { struct sockaddr_in address;
 #ifdef __WINDOWS__
-    DWORD len = sizeof(address);
+    int len = sizeof(address);
 #else
     socklen_t len = sizeof(address);
 #endif
