@@ -19,7 +19,7 @@
 
     You should have received a copy of the GNU Lesser General Public
     License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 :- module(test_call, [test_call/0]).
@@ -40,7 +40,8 @@ test_call :-
 		    cross_module_call,
 		    snip,
 		    no_autoload,
-		    setup_call_cleanup
+		    setup_call_cleanup,
+		    catch
 		  ]).
 
 :- begin_tests(call1).
@@ -54,6 +55,16 @@ test(clause, Body == call(X)) :-
 	clause(call1_a(X), Body).
 test(clause, Body == call(X)) :-
 	clause(call1_b(X), Body).
+test(big_clause, V==N) :-
+	N = 100000,
+	trim_stacks,			% ensure shifts are needed
+        link_clause(N, 0, V, Body),
+	call(Body).
+
+link_clause(1, V0, V, succ(V0, V)) :- !.
+link_clause(N, V0, V, (succ(V0, V1), G)) :-
+        N2 is N - 1,
+        link_clause(N2, V1, V, G).
 
 :- end_tests(call1).
 
@@ -69,6 +80,31 @@ test(error, error(type_error(callable, 1))) :-
 
 test(error, error(type_error(callable, 1))) :-
 	call(1, a, b).
+test(if_then_else, all(X==[a])) :-		% Not that I like it ...
+	call(;(true->X=a),
+	     X=b).
+test(disj, all(X==[a,b])) :-
+	call(;(X=a),
+	     X=b).
+test(conj, all(X==[])) :-
+	call(','(X=a),
+	     X=b).
+test(qn8, error(existence_error(procedure, call8:does_not_exist/8))) :-
+	Goal = call(call8:does_not_exist, a, b, c, d, e, f, g, h),
+	Goal.
+test(an8, error(existence_error(procedure, _:does_not_exist/8))) :-
+	Goal = call(does_not_exist, a, b, c, d, e, f, g, h),
+	Goal.
+test(qe8, true) :-
+	Goal = call(call8:exists, a, b, c, d, e, f, g, h),
+	Goal.
+test(ae8, true) :-
+	Goal = call(exists, a, b, c, d, e, f, g, h),
+	Goal.
+
+call8:exists(a, b, c, d, e, f, g, h).
+exists(a, b, c, d, e, f, g, h).
+exists(a,b).
 
 :- end_tests(callN).
 
@@ -126,6 +162,18 @@ test(indent, [all(Gnosc == [scio])]) :-
 verum :-
 	( false *-> true ; true ).
 
+test(cut) :-
+	( !, fail ; true ) *-> fail ; true.
+
+test(cut2) :-
+	no_cut_cond.
+
+no_cut_cond :- (!, fail) *-> true.
+no_cut_cond.
+
+test(cut3) :-				% success handling (wrong C_SCUT decl)
+	true *-> true.
+
 :- end_tests(snip).
 
 
@@ -254,3 +302,11 @@ test_error_choice :-
 a(_).
 
 :- end_tests(setup_call_cleanup).
+
+:- begin_tests(catch).
+
+test(exit_nondet, fail) :-
+	catch((true;throw(homer_simpson(38))),_E,true),
+	fail.
+
+:- end_tests(catch).
