@@ -2,7 +2,7 @@
 	  [ test_expand/0
 	  ]).
 :- use_module(library(plunit)).
-:- set_base_module(user).
+:- set_module(base(user)).
 
 test_expand :-
 	run_tests([ expand
@@ -18,6 +18,8 @@ test_expand :-
 :- dynamic
 	user:term_expansion/2,
 	user:goal_expansion/2.
+:- discontiguous
+	goal_expansion/2.
 
 user:term_expansion(b2c, final_c).
 
@@ -41,6 +43,23 @@ a2b.					% chained expansion
 e_not :-
 	not(g_b).
 
+g_c.
+
+% Test meta-predicate declaration before (local) definition of the
+% predicate.
+
+goal_expansion(foo, bar).
+
+:- meta_predicate
+	run(0).
+
+test_foo_bar :-
+	run(foo).
+
+bar.
+
+run(Goal) :- Goal.
+
 
 		 /*******************************
 		 *	       TESTS		*
@@ -54,7 +73,9 @@ test(make_dcg, B == true) :-
 	clause(a(_,_), B).
 test(chained_term_expansion, B == true) :-
 	clause(final_c, B).
-test(meta_arg, B == not(g_c)) :-
-	clause(e_not, B).
+test(meta_arg, [fail]) :-
+	e_not.
+test(goal_expansion_local_pred) :-
+	test_foo_bar.
 
 :- end_tests(expand).
