@@ -19,7 +19,7 @@
 
     You should have received a copy of the GNU Lesser General Public
     License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 :- module(test_misc, [test_misc/0]).
@@ -38,7 +38,20 @@ test_misc :-
 
 :- begin_tests(misc).
 
+p(_).
+p(C) :- prolog_cut_to(C).
+
 test(read_only_flag, Access == read) :-
 	'$current_prolog_flag'(arch, _, _Global, Access, _Type).
+test(cut_to, all(X == [1])) :-
+	prolog_current_choice(Ch),
+	between(1, 5, X),
+	prolog_cut_to(Ch).
+test(cut_to, error(existence_error(choice,_))) :-
+	(   prolog_current_choice(C),
+	    p(C)
+	*-> fail
+	;   fail
+	).
 
 :- end_tests(misc).
