@@ -19,7 +19,7 @@
 
     You should have received a copy of the GNU Lesser General Public
     License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
 :- module(test_term, [test_term/0]).
@@ -136,9 +136,14 @@ test(ground, [sto(rational_trees), fail]) :- % Ulrich
 	A=[A|B], B=[A], D=[[A|B]|A],
 	A = [_,_], D = [_,_,_],
 	A =@= D.
-test(sharing_cycles, [sto(rational_trees), fail, fixme(sharing_cycles)]) :-
+test(sharing_cycles, [sto(rational_trees), fail]) :-
 	A=[A|B], C=[A|D], B=[A|E], F=[A|F], D=[F|E],
 	A =@= C.
+test(cycle_with_prefix, [sto(rational_trees), fail]) :-
+	A = [A|_],
+	X = [A|Y],
+	B = [X|Y],
+	A =@= B.
 
 v(_).
 
