@@ -236,7 +236,7 @@ void
 updateAlerted(PL_local_data_t *ld)
 { int mask = 0;
 
-  if ( ld->signal.pending )			mask |= ALERT_SIGNAL;
+  if ( is_signalled(ld) )			mask |= ALERT_SIGNAL;
 #ifdef O_PROFILE
   if ( ld->profile.active )			mask |= ALERT_PROFILE;
 #endif
@@ -263,9 +263,10 @@ updateAlerted(PL_local_data_t *ld)
 int
 raiseSignal(PL_local_data_t *ld, int sig)
 { if ( sig > 0 && sig <= MAXSIGNAL && ld )
-  { simpleMutexLock(&ld->signal.sig_lock);
-    ld->signal.pending |= ((int64_t)1 << (sig-1));
-    simpleMutexUnlock(&ld->signal.sig_lock);
+  { int off = (sig-1) / 32;
+    int mask = (1 << ((sig-1)%32));
+
+    __sync_or_and_fetch(&ld->signal.pending[off], mask);
     updateAlerted(ld);
     return TRUE;
   }
@@ -274,13 +275,6 @@ raiseSignal(PL_local_data_t *ld, int sig)
 }
 
 
-static inline int
-is_signalled(ARG1_LD)
-{ return unlikely(LD->signal.pending != 0);
-}
-
-
-
 		 /*******************************
 		 *	   STACK-LAYOUT		*
 		 *******************************/
